@@ -101,6 +101,14 @@ NAMESPACE_ARRAYS = {
     },
 }
 
+try:
+    import jax
+
+    # enable double-precision
+    jax.config.update("jax_enable_x64", True)
+except ImportError:
+    pass
+
 
 class _BaseTest:
     def setup_for_test(self, request, namespace):
@@ -155,7 +163,7 @@ class TestTopLevelMethods(_BaseTest):
         assert isinstance(result.data, self.Array)
 
     def test_merge(self):
-        result = xr.merge([self.x1, self.x2], compat="override")
+        result = xr.merge([self.x1, self.x2], compat="override", join="outer")
         assert isinstance(result.foo.data, self.Array)
 
     def test_where(self):
